Unit EditAlign;
{************************************************************************}
{DESC: TEditAlign is an improved TEdit component with alignment          }
{      (that handles left, center or right text justification),          }
{      ExitOnEnterKey and Multiline properties.                          }
{                                                                        }
{   (c) Ryan Fischbach                                                   }
{   Version 1.1, March 16, 1999                                          }
{                                                                        }
{   This component is freeware                                           }
{   Any change to this component must be notified to the author:         }
{                                                                        }
{   ryantf@bigfoot.com                                                   }
{                                                                        }
{                                                                        }
{   Many thanks to Jos Mara Ferri for this great observation...        }
{     Alignment only works if the control is a multiline control,        }
{     so if multiline property is false, alignment is always left :-(    }
{************************************************************************}

{------------------------------------------------------------------------}
{-------------------------}    INTERFACE   {-----------------------------}
{------------------------------------------------------------------------}

Uses
  Messages, Windows, SysUtils, Classes, StdCtrls, Controls;


Type
  TEditAlign = class (TCustomEdit)
    private
      fAlignment: TAlignment;
      bExitOnEnterKey: boolean;
      bMultiline: boolean;
      bWordWrap: boolean;
    public
      Constructor Create(aOwner:tComponent); override;
      Destructor Destroy; override;
    protected
      procedure CreateParams(var Params:tCreateParams); override;
      procedure CMEnter(var iMessage:tCMEnter); message CM_ENTER;
      procedure KeyPress(var Key:char); override;
      procedure SetAlignment(Value:tAlignment);
      procedure SetExitOnEnterKey(Value:boolean);
      procedure SetMultiline(Value:boolean);
      procedure SetWordWrap(Value:boolean);
      procedure PurgeTextOfCRLF;
    published
      property AutoSelect;
      property AutoSize;
      property BorderStyle;
      property CharCase;
      property Color;
      property Ctl3D;
      property DragCursor;
      property DragMode;
      property Enabled;
      property Font;
      property HideSelection;
      property MaxLength;
      property OEMConvert;
      property ParentColor;
      property ParentCtl3D;
      property ParentFont;
      property ParentShowHint;
      property PasswordChar;
      property PopupMenu;
      property ReadOnly;
      property ShowHint;
      property TabOrder;
      property TabStop;
      property Text;
      property Visible;
      property OnChange;
      property OnClick;
      property OnDblClick;
      property OnDragDrop;
      property OnDragOver;
      property OnEndDrag;
      property OnEnter;
      property OnExit;
      property OnKeyDown;
      property OnKeyPress;
      property OnKeyUp;
      property OnMouseDown;
      property OnMouseMove;
      property OnMouseUp;
      property OnStartDrag;

      property Alignment: TAlignment read fAlignment write SetAlignment default taLeftJustify;
      property ExitOnEnterKey: boolean read bExitOnEnterKey write SetExitOnEnterKey default false;
      property MultiLine: Boolean read bMultiline write SetMultiline default false;
      property WordWrap: Boolean read bWordWrap write SetWordWrap default false;
  end;

procedure Register;

{------------------------------------------------------------------------}
{-------------------------} IMPLEMENTATION {-----------------------------}
{------------------------------------------------------------------------}


procedure Register;
{************************************************************************}
{DESC: Register this component with Delphi.                              }
{************************************************************************}
Begin
  RegisterComponents('Standard', [TEditAlign]);
End;


Constructor tEditAlign.Create(aOwner:tComponent);
{************************************************************************}
{DESC: Constructor.                                                      }
{************************************************************************}
Begin
  inherited Create(AOwner);
  fAlignment := taLeftJustify;
  bMultiline := false;
  bWordWrap := false;
End;

Destructor tEditAlign.Destroy;
{************************************************************************}
{DESC: Destructor.                                                       }
{************************************************************************}
Begin
  inherited Destroy;
End;

procedure tEditAlign.CreateParams(var Params:tCreateParams);
{************************************************************************}
{DESC: Allow windows to handle the alignment of the field.               }
{************************************************************************}
Begin
  inherited CreateParams(Params);
  Params.Style := Params.Style or LongWord(ES_MultiLine);
  case fAlignment of
    taLeftJustify:
      Params.Style := Params.Style or LongWord(ES_Left);
    taRightJustify:
      Params.Style := Params.Style or LongWord(ES_Right);
    else {taCenter}
      Params.Style := Params.Style or LongWord(ES_Center);
  end;{case}
  if bWordWrap then
    Params.Style := Params.Style and (not LongWord(ES_AUTOHSCROLL))
  else
    Params.Style := Params.Style and (not LongWord(0));
End;

procedure tEditAlign.CMEnter(var iMessage:tCMEnter);
Begin
  if AutoSelect then
    SelectAll;
  inherited;
End;

procedure tEditAlign.KeyPress(var Key:char);
Begin
  case ord(Key) of
    VK_Return:
      begin
        if bExitOnEnterKey then
          begin
            Key := #0;
            PostMessage(Handle,WM_KEYDOWN,VK_TAB,0);
          end
        else
          if (not bMultiline) then
            Key := #0;
      end;
  end;{case}
  inherited KeyPress(Key);
End;

procedure tEditAlign.SetAlignment(Value:tAlignment);
Begin
  if fAlignment<>Value then
    begin
      fAlignment := Value;
      RecreateWnd;
    end;
End;

procedure tEditAlign.SetExitOnEnterKey(Value:boolean);
Begin
  if bExitOnEnterKey<>Value then
    begin
      bExitOnEnterKey := Value;
      if bExitOnEnterKey and MultiLine then
        MultiLine := false
      else
        RecreateWnd;
    end;
End;

procedure tEditAlign.SetMultiline(Value:boolean);
Begin
  if bMultiline<>Value then
    begin
      bMultiline := Value;
      if bMultiLine then
        bExitOnEnterKey := false
      else
        PurgeTextOfCRLF;
    end;
End;

procedure tEditAlign.SetWordWrap(Value:boolean);
Begin
  if bWordWrap<>Value then
    begin
      bWordWrap := Value;
      RecreateWnd;
    end;
End;

procedure tEditAlign.PurgeTextOfCRLF;
Var
  TmpPos: integer;
  TmpStr: string;
Begin
  TmpStr := Text;
  TmpPos := pos(#13,TmpStr);
  while (TmpPos<>0) do begin
    delete(TmpStr,TmpPos,2);
    TmpPos := pos(#13,TmpStr);
  end;{while}
  Text := TmpStr;
End;


End.
